#Embedded file name: carbon/common/script/net\xmlrpcClient.py
"""
A simple service serving xmlrpc proxies for clients of the service.
It maintains a cache of live connections so using this is faster than
creating a fresh proxy each time a call needs to be made.
"""
import service
import contextlib
import xmlrpclib
import urlparse
import socket
import collections
import locks
from service import ROLE_SERVICE

class CachedMultiMap(object):

    def __init__(self, highWater = 10, lowWater = None):
        self.map = collections.defaultdict(list)
        self.times = {}
        self.time = 0
        self.highWater = highWater
        if lowWater is None:
            lowWater = self.highWater / 2
        self.lowWater = min(lowWater, self.highWater)

    def __len__(self):
        return len(self.times)

    def Insert(self, key, val):
        time = self.time
        self.time += 1
        e = (time, val)
        self.map[key].append(e)
        self.times[time] = key
        if len(self.times) > self.highWater:
            self.Trim(self.lowWater)

    def Get(self, key):
        l = self.map[key]
        t0, v = l[-1]
        t = self.time
        self.time += 1
        del self.times[t0]
        self.times[t] = key
        l[-1] = (t, v)
        return v

    def Pop(self, key, first = False):
        l = self.map[key]
        try:
            t0, v = l.pop(0 if first else -1)
        except IndexError:
            raise KeyError, key

        del self.times[t0]
        if not l:
            del self.map[key]
        return v

    def Trim(self, max):
        times = self.times
        if len(times) > max:
            k = self.times.keys()
            k.sort()
            k = k[:-max]
            Pop = self.Pop
            for t in k:
                Pop(times[t], True)

    def Clear(self):
        self.map = collections.defaultdict(list)
        self.times = {}
        self.time = self.n = 0


class RetryProxyMethod:

    def __init__(self, send, name, retries = 3):
        self.__send = send
        self.__name = name
        self.__retries = retries

    def __getattr__(self, name):
        return RetryProxyMethod(self.__send, '.'.join((self.__name, name)), self.__retries)

    def __call__(self, *args):
        for i in xrange(self.__retries):
            try:
                return self.__send(self.__name, args)
            except socket.error:
                if i == self.__retries - 1:
                    raise


class RetryProxy(xmlrpclib.ServerProxy):
    """A class that will retry a specified number of times if we get a socket error"""
    __guid__ = 'xmlrpc.RetryProxy'

    def __init__(self, uri, transport = None, encoding = None, verbose = 0, allow_none = 0, use_datetime = 0, retries = 3):
        xmlrpclib.ServerProxy.__init__(self, uri, transport, encoding, verbose, allow_none, use_datetime)
        self.__retries = retries

    def __getattr__(self, name):
        return RetryProxyMethod(self._ServerProxy__request, name, self.__retries)


class XmlrpcClient(service.Service):
    __exportedcalls__ = {'GetProxy': [ROLE_SERVICE],
     'ReturnProxy': [ROLE_SERVICE],
     'Proxy': [ROLE_SERVICE]}
    __guid__ = 'svc.xmlrpcClient'
    __dependencies__ = []

    def Run(self, memStream = None):
        """
        Start the xmlrpcService.
        """
        self.proxiesByKey = CachedMultiMap(10)
        self.dnsMap = {}

    def Stop(self, memStream):
        """
        Stop service
        """
        self.proxiesByKey.Clear()

    def GetProxy(self, uri, proxyClass = xmlrpclib.ServerProxy, proxyArgs = ()):
        key = (uri, proxyClass, proxyArgs)
        try:
            return (key, self.proxiesByKey.Pop(key))
        except KeyError:
            uri = self.FixUri(uri)
            return (key, proxyClass(uri, *proxyArgs))

    def ReturnProxy(self, key, proxy, discard = False):
        """Return the proxy back into the cache.  The 'discard' flag signals that the
        proxy has become 'bad' somehow
        """
        if not discard:
            self.proxiesByKey.Insert(key, proxy)

    def FixUri(self, uri):
        """cache DNS lookups on the uri to make it faster"""
        parts = urlparse.urlsplit(uri)
        key = parts[1]
        try:
            while True:
                netloc = self.dnsMap[key]
                if isinstance(netloc, locks.Event):
                    netloc.wait()
                else:
                    break

        except KeyError:
            event = locks.Event()
            self.dnsMap[key] = event
            try:
                netloc = key
                hostport = netloc.split(':')
                if len(hostport) == 2:
                    host, port = hostport
                else:
                    host, port = hostport[0], None
                try:
                    info = socket.getaddrinfo(host, port)
                    info = [ e for e in info if e[0] == socket.AF_INET ]
                    if info:
                        if port is not None:
                            host, port = info[0][4]
                            netloc = host + ':' + str(port)
                        else:
                            netloc = host
                except socket.gaierror:
                    pass

            except:
                del self.dnsMap[key]
            else:
                self.dnsMap[key] = netloc
            finally:
                event.set()

        if netloc != parts[1]:
            uri = urlparse.urlunsplit((parts[0],) + (netloc,) + parts[2:])
        return uri

    def Call(self, uri, method, *args, **kw):
        with self.ProxyContext(uri, kw.get('proxyClass', xmlrpclib.ServerProxy), kw.get('proxyArgs', ())) as p:
            return getattr(p, method)(*args)

    @contextlib.contextmanager
    def Proxy(self, uri, proxyClass = xmlrpclib.ServerProxy, proxyArgs = ()):
        """ A context in which is is handy to call objects.  Can provide error handling """
        k, p = self.GetProxy(uri, proxyClass, proxyArgs)
        discard = False
        try:
            yield p
        except socket.error:
            discard = True
            raise
        finally:
            self.ReturnProxy(k, p, discard)
