#Embedded file name: carbonui/primitives\sprite.py
from .base import Base
import carbonui.const as uiconst
import remotefilecache
import trinity
import types
import weakref
import uthread
try:
    import GameWorld
except:
    GameWorld = None

DEFAULTCOLOR = (1.0,
 1.0,
 1.0,
 1.0)

class VisibleBase(Base):
    """
    A base class for visible UI objects (basically everything but containers).
    """
    __guid__ = 'uiprimitives.VisibleBase'
    __renderObject__ = None
    default_name = 'VisibleBase'
    default_left = 0
    default_top = 0
    default_color = DEFAULTCOLOR
    default_opacity = None
    default_blendMode = trinity.TR2_SBM_BLEND
    default_depth = 0.0
    _color = None

    def ApplyAttributes(self, attributes):
        Base.ApplyAttributes(self, attributes)
        color = attributes.get('color', self.default_color)
        if type(color) == types.TupleType and color != DEFAULTCOLOR:
            self.SetRGB(*color)
        self.blendMode = attributes.get('blendMode', self.default_blendMode)
        self.depth = attributes.get('depth', self.default_depth)
        opacity = attributes.get('opacity', self.default_opacity)
        if opacity is not None:
            self.opacity = opacity

    def Close(self):
        self._color = None
        Base.Close(self)

    @apply
    def blendMode():
        doc = '\n        Determines how the sprite blends with the background. Must be a \n        trinity.TR2_SBM_{X} constant'

        def fget(self):
            return self._blendMode

        def fset(self, value):
            self._blendMode = value
            ro = self.renderObject
            if ro:
                ro.blendMode = value

        return property(**locals())

    @apply
    def color():
        doc = 'Set color as (r,g,b,a) [0.0 - 1.0]'

        def fget(self):
            if self._color is None:
                self._color = PyColor(self)
            return self._color

        def fset(self, value):
            if type(value) == types.TupleType:
                self.SetRGB(*value)
            elif isinstance(value, PyColor):
                self._color = value
            ro = self.renderObject
            if ro:
                ro.color = self._color.GetRGBA()

        def fdel(self):
            self._color = None

        return property(**locals())

    @apply
    def opacity():
        doc = 'opacity [0.0 - 1.0]'

        def fget(self):
            return self.color.a

        def fset(self, value):
            self.color.a = value

        return property(**locals())

    @apply
    def depth():
        doc = 'z-axis offset. Only has meaning when using with 3d rendering'

        def fget(self):
            return self._depth

        def fset(self, value):
            self._depth = value
            ro = self.renderObject
            if ro:
                ro.depth = value

        return property(**locals())

    def SetRGB(self, *color):
        """
        Set sprite color tuple (r,g,b,a) [0.0 - 1.0]
        """
        if type(color) != types.TupleType:
            return
        self.color.SetRGB(*color)

    SetRGBA = SetRGB

    def GetRGB(self):
        return self.color.GetRGB()

    def GetRGBA(self):
        return self.color.GetRGBA()

    def SetAlpha(self, alpha):
        self.color.SetAlpha(alpha)

    def GetAlpha(self):
        return self.color.GetAlpha()


class TexturedBase(VisibleBase):
    """
    A base UI texture object for others to inherit from. The class holds two textures;
    self.texture and self.textureSecondary. The two are then blended together depending
    on the spriteEffect used.
    """
    __guid__ = 'uiprimitives.TexturedBase'
    __renderObject__ = None
    default_noScale = 0
    default_texturePath = None
    default_textureSecondaryPath = None
    default_glowFactor = 0.0
    default_glowExpand = 0.0
    default_glowColor = (1, 1, 1, 1)
    default_shadowOffset = (0, 0)
    default_shadowColor = (0, 0, 0, 0.5)
    default_spriteEffect = trinity.TR2_SFX_COPY
    default_rotation = 0.0
    default_tileX = False
    default_tileY = False
    _textureSecondary = None

    def ApplyAttributes(self, attributes):
        VisibleBase.ApplyAttributes(self, attributes)
        self.texture = trinity.Tr2Sprite2dTexture()
        texturePath = attributes.get('texturePath', self.default_texturePath)
        if texturePath:
            self.SetTexturePath(texturePath)
        secondaryPath = attributes.get('textureSecondaryPath', self.default_textureSecondaryPath)
        if secondaryPath:
            self.SetSecondaryTexturePath(secondaryPath)
        self.glowFactor = attributes.get('glowFactor', self.default_glowFactor)
        self.glowExpand = attributes.get('glowExpand', self.default_glowExpand)
        self.glowColor = attributes.get('glowColor', self.default_glowColor)
        self.shadowOffset = attributes.get('shadowOffset', self.default_shadowOffset)
        self.shadowColor = attributes.get('shadowColor', self.default_shadowColor)
        rectLeft = attributes.rectLeft
        if rectLeft:
            self.rectLeft = rectLeft
        rectTop = attributes.rectTop
        if rectTop:
            self.rectLeft = rectTop
        rectWidth = attributes.rectWidth
        if rectWidth:
            self.rectWidth = rectWidth
        rectHeight = attributes.rectHeight
        if rectHeight:
            self.rectHeight = rectHeight
        self.spriteEffect = attributes.get('spriteEffect', self.default_spriteEffect)
        self.rotation = attributes.get('rotation', self.default_rotation)
        self.tileX = attributes.get('tileX', self.default_tileX)
        self.tileY = attributes.get('tileY', self.default_tileY)

    def Close(self):
        self._texture = None
        self._textureSecondary = None
        VisibleBase.Close(self)

    @apply
    def spriteEffect():
        doc = '\n        determines how the primary and secondary textures are blended together. Must be\n        a trinity.TR2_SFX_{X} constant'

        def fget(self):
            return self._spriteEffect

        def fset(self, value):
            self._spriteEffect = value
            ro = self.renderObject
            if ro and hasattr(ro, 'spriteEffect'):
                ro.spriteEffect = value

        return property(**locals())

    @apply
    def texture():
        doc = 'The primary texture. An instance of trinity.Tr2Sprite2dTexture'

        def fget(self):
            return self._texture

        def fset(self, value):
            self._texture = value
            ro = self.renderObject
            if ro:
                ro.texturePrimary = self._texture

        return property(**locals())

    @apply
    def textureSecondary():
        doc = 'The secondary texture. An instance of trinity.Tr2Sprite2dTexture'

        def fget(self):
            return self._textureSecondary

        def fset(self, value):
            self._textureSecondary = value
            ro = self.renderObject
            if ro and value is not None:
                ro.textureSecondary = value

        return property(**locals())

    @apply
    def rectLeft():
        doc = 'Manually set left edge of primary texture'

        def fget(self):
            return self._rectLeft

        def fset(self, value):
            self._rectLeft = value
            ro = self.renderObject
            if ro and ro.texturePrimary:
                ro.texturePrimary.srcX = value

        return property(**locals())

    @apply
    def rectTop():
        doc = 'Manually set top edge of primary texture'

        def fget(self):
            return self._rectTop

        def fset(self, value):
            self._rectTop = value
            ro = self.renderObject
            if ro and ro.texturePrimary:
                ro.texturePrimary.srcY = value

        return property(**locals())

    @apply
    def rectWidth():
        doc = 'Manually set width of primary texture'

        def fget(self):
            return self._rectWidth

        def fset(self, value):
            self._rectWidth = value
            ro = self.renderObject
            if ro and ro.texturePrimary:
                ro.texturePrimary.srcWidth = value

        return property(**locals())

    @apply
    def rectHeight():
        doc = 'Manually set height of primary texture'

        def fget(self):
            return self._rectHeight

        def fset(self, value):
            self._rectHeight = value
            ro = self.renderObject
            if ro and ro.texturePrimary:
                ro.texturePrimary.srcHeight = value

        return property(**locals())

    @apply
    def rectSecondary():
        doc = 'Manually set rect (srcX, srcY, srcWidth, srcHeight) of secondary texture'

        def fget(self):
            if self.textureSecondary:
                return (self.textureSecondary.srcX,
                 self.textureSecondary.srcY,
                 self.textureSecondary.srcWidth,
                 self.textureSecondary.srcHeight)

        def fset(self, value):
            if self.textureSecondary:
                self.textureSecondary.srcX, self.textureSecondary.srcY, self.textureSecondary.srcWidth, self.textureSecondary.srcHeight = value

        return property(**locals())

    @apply
    def translationSecondary():
        doc = 'Manually set translation of secondary texture'

        def fget(self):
            if self.textureSecondary:
                return self.textureSecondary.translation

        def fset(self, value):
            if self.textureSecondary:
                self.textureSecondary.translation = value
                if value != (0, 0):
                    self.__EnableSecondaryTransform()

        return property(**locals())

    @apply
    def rotationSecondary():
        doc = 'Manually set rotation of secondary texture'

        def fget(self):
            if self.textureSecondary:
                return self.textureSecondary.rotation

        def fset(self, value):
            if self.textureSecondary:
                self.textureSecondary.rotation = value
                if value != 0.0:
                    self.__EnableSecondaryTransform()

        return property(**locals())

    @apply
    def scaleSecondary():
        doc = 'Manually set scale of secondary texture'

        def fget(self):
            if self.textureSecondary:
                return self.textureSecondary.scale

        def fset(self, value):
            if self.textureSecondary:
                self.textureSecondary.scale = value
                if value != (1.0, 1.0):
                    self.__EnableSecondaryTransform()

        return property(**locals())

    @apply
    def scaleSecondary():
        doc = 'Manually set scaling of secondary texture'

        def fget(self):
            if self.textureSecondary:
                return self.textureSecondary.scale

        def fset(self, value):
            if self.textureSecondary:
                self.textureSecondary.scale = value
                if value != (1.0, 1.0):
                    self.textureSecondary.useTransform = True
                    self._EnableStandAloneTexture(self.textureSecondary)

        return property(**locals())

    @apply
    def scalingCenterSecondary():
        doc = 'Manually set scale center of secondary texture'

        def fget(self):
            if self.textureSecondary:
                return self.textureSecondary.scalingCenter

        def fset(self, value):
            if self.textureSecondary:
                self.textureSecondary.scalingCenter = value

        return property(**locals())

    @apply
    def glowFactor():
        doc = 'Glow effect amount [0.0 - 1.0]'

        def fget(self):
            return self._glowFactor

        def fset(self, value):
            self._glowFactor = value
            ro = self.renderObject
            if ro:
                ro.glowFactor = value

        return property(**locals())

    @apply
    def glowExpand():
        doc = 'Glow effect expand [0.0 - inf]'

        def fget(self):
            return self._glowExpand

        def fset(self, value):
            self._glowExpand = value
            ro = self.renderObject
            if ro:
                ro.glowExpand = value

        return property(**locals())

    @apply
    def glowColor():
        doc = 'Glow effect color tuple (r,g,b,a) [0.0 - 1.0]'

        def fget(self):
            return self._glowColor

        def fset(self, value):
            self._glowColor = value
            ro = self.renderObject
            if ro:
                ro.glowColor = value

        return property(**locals())

    @apply
    def shadowOffset():
        doc = 'Shadow effect offset tuple (x,y) [0.0 - inf]'

        def fget(self):
            return self._shadowOffset

        def fset(self, value):
            self._shadowOffset = value
            ro = self.renderObject
            if ro:
                ro.shadowOffset = value

        return property(**locals())

    @apply
    def shadowColor():
        doc = 'Shadow effect color tuple (r,g,b,a) [0.0 - 1.0]'

        def fget(self):
            return self._shadowColor

        def fset(self, value):
            self._shadowColor = value
            ro = self.renderObject
            if ro:
                ro.shadowColor = value

        return property(**locals())

    @apply
    def rotation():
        doc = 'Set rotation of primary texture'

        def fget(self):
            if self.texture:
                return self.texture.rotation
            else:
                return 0

        def fset(self, value):
            if self.texture:
                self.texture.rotation = value
                if value != 0.0:
                    self.texture.useTransform = True
                    self._EnableStandAloneTexture(self.texture)

        return property(**locals())

    @apply
    def scale():
        doc = 'Manually set scaling of primary texture'

        def fget(self):
            if self.texture:
                return self.texture.scale

        def fset(self, value):
            if self.texture:
                self.texture.scale = value
                if value != (1.0, 1.0):
                    self.texture.useTransform = True
                    self._EnableStandAloneTexture(self.texture)

        return property(**locals())

    @apply
    def scalingCenter():
        doc = 'Manually set scale center of primary texture'

        def fget(self):
            if self.texture:
                return self.texture.scalingCenter

        def fset(self, value):
            if self.texture:
                self.texture.scalingCenter = value

        return property(**locals())

    @apply
    def tileX():
        doc = 'Set whether the texture should tile on X-axis'

        def fget(self):
            if self.texture:
                return self.texture.tileX

        def fset(self, value):
            if self.texture:
                self.texture.tileX = value
                if value:
                    self.texture.useTransform = True
                    self._EnableStandAloneTexture(self.texture)

        return property(**locals())

    @apply
    def tileY():
        doc = 'Set whether the texture should tile on Y-axis'

        def fget(self):
            if self.texture:
                return self.texture.tileY

        def fset(self, value):
            if self.texture:
                self.texture.tileY = value
                if value:
                    self.texture.useTransform = True
                    self._EnableStandAloneTexture(self.texture)

        return property(**locals())

    @apply
    def tileXSecondary():
        doc = 'Set whether the texture should tile on X-axis on secondary texture'

        def fget(self):
            if self.textureSecondary:
                return self.textureSecondary.tileX

        def fset(self, value):
            if self.textureSecondary:
                self.textureSecondary.tileX = value
                if value:
                    self.textureSecondary.useTransform = True
                    self._EnableStandAloneTexture(self.textureSecondary)

        return property(**locals())

    @apply
    def tileYSecondary():
        doc = 'Set whether the texture should tile on Y-axis on secondary texture'

        def fget(self):
            if self.textureSecondary:
                return self.textureSecondary.tileY

        def fset(self, value):
            if self.textureSecondary:
                self.textureSecondary.tileY = value
                if value:
                    self.textureSecondary.useTransform = True
                    self._EnableStandAloneTexture(self.textureSecondary)

        return property(**locals())

    def _EnableStandAloneTexture(self, texture):
        if getattr(texture, 'atlasTexture', None) is not None:
            texture.atlasTexture.isStandAlone = True

    def __EnableSecondaryTransform(self):
        self.textureSecondary.useTransform = True
        self._EnableStandAloneTexture(self.textureSecondary)

    def ReloadTexture(self):
        """
        Force a reload of the primary texture
        """
        if self.texture:
            if self.texture.atlasTexture:
                self.texture.atlasTexture.Reload()

    def ReloadSecondaryTexture(self):
        """
        Force a reload of the secondary texture
        """
        if self.textureSecondary:
            if self.textureSecondary.atlasTexture:
                self.textureSecondary.atlasTexture.Reload()

    def SetTexturePath(self, texturePath):
        """
        Set primary texture path
        """
        if self.texture:
            newPath = str(texturePath or '')
            if newPath != self.texture.resPath:
                self.texture.resPath = newPath

    LoadTexture = SetTexturePath

    def GetTexturePath(self):
        """
        Get primary texture path
        """
        if self.texture:
            return self.texture.resPath

    texturePath = property(GetTexturePath, SetTexturePath)

    def SetSecondaryTexturePath(self, texturePath):
        """
        Set secondary texture path
        """
        if not self.textureSecondary:
            self.textureSecondary = trinity.Tr2Sprite2dTexture()
        self.textureSecondary.resPath = str(texturePath or '')

    def GetSecondaryTexturePath(self):
        """
        Get secondary texture path
        """
        if self.textureSecondary:
            return self.textureSecondary.resPath

    def SetRect(self, rectLeft, rectTop, rectWidth, rectHeight):
        self.rectLeft = rectLeft
        self.rectTop = rectTop
        self.rectWidth = rectWidth
        self.rectHeight = rectHeight

    def LoadIcon(self, iconNo, ignoreSize = False):
        """Takes iconNo in form of "ui_1_16_12". "ui_" maps this to file 1_16_12 in res/ui/texture/icons.
        The numbers in the string are legacy code, but the mid part is still relevant since its used
        to identify the size of the icon. Client classes can extend this function by supporting more then
        "ui_" prefix.        
        icon can also be full resPath or cachePath
        ignoreSize defines if the actual size of the icon (16 in this case) will be applied to the sprite.
        Returns the iconNo if not resolved by this function
        """
        if self.destroyed:
            return
        if iconNo.startswith('res:') or iconNo.startswith('cache:'):
            self.LoadTexture(iconNo)
            return
        if iconNo.startswith('ui_'):
            root, sheetNo, iconSize, icon = iconNo.split('_')
            resPath = 'res:/ui/texture/icons/%s_%s_%s.png' % (int(sheetNo), int(iconSize), int(icon))
            iconSize = int(iconSize)
            self.LoadTexture(resPath)
            if not ignoreSize and self.GetAlign() != uiconst.TOALL and self.texture.atlasTexture:
                self.width = iconSize
                self.height = iconSize
            return
        return iconNo


class Sprite(TexturedBase):
    """
    A basic UI sprite for rendering a texture to screen
    """
    __guid__ = 'uiprimitives.Sprite'
    __renderObject__ = trinity.Tr2Sprite2d
    isDragObject = True
    default_noScale = 0
    default_left = 0
    default_top = 0
    default_width = 0
    default_height = 0
    default_pickRadius = 0
    default_saturation = 1.0
    default_effectOpacity = 1.0
    default_useSizeFromTexture = False

    def ApplyAttributes(self, attributes):
        TexturedBase.ApplyAttributes(self, attributes)
        self.pickRadius = attributes.get('pickRadius', self.default_pickRadius)
        self.saturation = attributes.get('saturation', self.default_saturation)
        self.effectOpacity = attributes.get('effectOpacity', self.default_effectOpacity)
        self.renderObject.useSizeFromTexture = attributes.get('useSizeFromTexture', self.default_useSizeFromTexture)

    @apply
    def pickRadius():
        doc = 'Pick radius'

        def fget(self):
            return self._pickRadius

        def fset(self, value):
            self._pickRadius = value
            ro = self.renderObject
            if ro and hasattr(ro, 'pickRadius'):
                if value < 0:
                    ro.pickRadius = value
                else:
                    ro.pickRadius = uicore.ScaleDpi(value) or 0.0

        return property(**locals())

    @apply
    def useSizeFromTexture():
        doc = '\n            If set, displayWidth/Height are ignored and size of primary texture is used instead.\n            '

        def fget(self):
            return self._useSizeFromTexture

        def fset(self, value):
            self._useSizeFromTexture = value
            if self.renderObject:
                self.renderObject.useSizeFromTexture = value

        return property(**locals())

    @apply
    def saturation():
        doc = '\n        Saturation of texture when using TR2_SFX_COLOROVERLAY or TR2_SFX_SOFTLIGHT.\n        '

        def fget(self):
            return self._saturation

        def fset(self, value):
            self._saturation = value
            ro = self.renderObject
            if ro:
                ro.saturation = value or 0.0

        return property(**locals())

    @apply
    def effectOpacity():
        doc = '\n        Opacity of effect when using TR2_SFX_COLOROVERLAY or TR2_SFX_SOFTLIGHT.\n        '

        def fget(self):
            return self._effectOpacity

        def fset(self, value):
            self._effectOpacity = value
            ro = self.renderObject
            if ro:
                ro.effectOpacity = value or 0.0

        return property(**locals())


class VideoSprite(Sprite):
    """
    A UI object capable of playing .bnk video files
    To support alpha, use a .bik file with an alpha plane, and set spriteEffect to trinity.TR2_SFX_COPY
    """
    __guid__ = 'uiprimitives.VideoSprite'
    __renderObject__ = trinity.Tr2Sprite2d
    __notifyevents__ = ['OnAudioActivated']
    default_videoPath = ''
    default_videoLoop = False
    default_videoAutoPlay = True
    default_muteAudio = False
    default_spriteEffect = trinity.TR2_SFX_NOALPHA

    def Close(self, *args, **kwds):
        self.Pause()
        uicore.uilib.GetVideoJob().steps.fremove(self._updateStep)
        if self.positionComponent:
            self.positionComponent.UnRegisterPlacementObserverWrapper(self.positionObserver)
            self.positionComponent = None
            self.positionObserver = None
        self.emitter = None
        self._updateStep = None
        Sprite.Close(self, *args, **kwds)

    def ApplyAttributes(self, attributes):
        Sprite.ApplyAttributes(self, attributes)
        sm.RegisterNotify(self)
        self._updateStep = None
        self._isFetchingFile = False
        RO = self.GetRenderObject()
        if 'videoPath' in attributes:
            tex = trinity.Tr2Sprite2dBinkTexture()
            tempPositionComponent = attributes.get('positionComponent', None)
            self.emitter, outputChannel = uicore.audioHandler.GetAudioBus(is3D=tempPositionComponent is not None)
            tex.outputChannel = outputChannel
            self.positionComponent = self.SetPositionComponent(tempPositionComponent)
            videoPath = attributes.get('videoPath', self.default_videoPath)
            self._SetVideoResPathWithPrefetch(tex, videoPath)
            if 'pos' in attributes:
                pos = attributes.get('pos', (self.default_left,
                 self.default_top,
                 self.default_width,
                 self.default_height))
                RO.displayX, RO.displayY, RO.displayWidth, RO.displayHeight = pos
            tex.loop = attributes.get('videoLoop', self.default_videoLoop)
            if attributes.get('muteAudio', self.default_muteAudio):
                tex.MuteAudio()
            tex.Play()
            self.texture = tex
            if attributes.get('videoAutoPlay', self.default_videoAutoPlay):
                self._updateStep = uicore.uilib.GetVideoJob().Update(RO.texturePrimary)

    def OnAudioActivated(self):
        """
        Restart the audio stuff because the initial event got lost.
        """
        if not self.texture:
            return
        self.Pause()
        if self._updateStep:
            uicore.uilib.GetVideoJob().steps.fremove(self._updateStep)
            self._updateStep = None
        self.emitter = None
        self.emitter, self.texture.outputChannel = uicore.audioHandler.GetAudioBus(is3D=self.positionComponent is not None)
        self.SetVideoPath(self.texture.resPath)
        RO = self.GetRenderObject()
        if RO:
            self._updateStep = uicore.uilib.GetVideoJob().Update(RO.texturePrimary)
        self.Play()

    def SetPositionComponent(self, positionComponent):
        if self.emitter and positionComponent and GameWorld:
            self.positionObserver = GameWorld.PlacementObserverWrapper(self.emitter)
            positionComponent.RegisterPlacementObserverWrapper(self.positionObserver)
            return positionComponent

    def GetVideoSize(self):
        RO = self.GetRenderObject()
        if RO and RO.texturePrimary.atlasTexture:
            return (RO.texturePrimary.atlasTexture.width, RO.texturePrimary.atlasTexture.height)

    def SetVideoPath(self, path):
        RO = self.GetRenderObject()
        if RO:
            self._SetVideoResPathWithPrefetch(RO.texturePrimary, path)

    def Play(self):
        RO = self.GetRenderObject()
        if RO:
            RO.texturePrimary.Play()
            if self._updateStep is None:
                self._updateStep = uicore.uilib.GetVideoJob().Update(RO.texturePrimary)

    def Pause(self):
        RO = self.GetRenderObject()
        if RO:
            RO.texturePrimary.Pause()

    def MuteAudio(self):
        RO = self.GetRenderObject()
        if RO:
            RO.texturePrimary.MuteAudio()

    def UnmuteAudio(self):
        RO = self.GetRenderObject()
        if RO:
            RO.texturePrimary.UnmuteAudio()

    @apply
    def isMuted():
        doc = ''

        def fget(self):
            RO = self.GetRenderObject()
            if RO:
                return RO.texturePrimary.isMuted

        return property(**locals())

    @apply
    def isPaused():
        doc = ''

        def fget(self):
            RO = self.GetRenderObject()
            if RO:
                return RO.texturePrimary.isPaused

        return property(**locals())

    @apply
    def isFinished():
        doc = ''

        def fget(self):
            if self.destroyed:
                return True
            if self._isFetchingFile:
                return False
            RO = self.GetRenderObject()
            if RO:
                return RO.texturePrimary.isFinished

        return property(**locals())

    @apply
    def videoFps():
        doc = ''

        def fget(self):
            RO = self.GetRenderObject()
            if RO:
                return RO.texturePrimary.videoFps

        return property(**locals())

    @apply
    def currentFrame():
        doc = ''

        def fget(self):
            RO = self.GetRenderObject()
            if RO:
                return RO.texturePrimary.currentFrame

        return property(**locals())

    def _SetVideoResPathWithPrefetch(self, tex, videoPath):
        """
        Sets the resPath attribute on the given Bink texture, prefetching the video
        file if needed. The prefetching will yield while the is downloaded.
        If the file is already resident, the prefetching returns immediately and
        the resPath is set.
        """

        def taskletfunc():
            remotefilecache.prefetch_single_file(videoPath)
            tex.resPath = videoPath
            tex.Play()
            self._isFetchingFile = False

        self._isFetchingFile = True
        uthread.new(taskletfunc)


class PyColor(object):
    """
    A color class owned by uiprimitives.Sprite. This class had to be added to provide
    backwards compatibility for old UI
    """

    def __init__(self, owner, r = 1.0, g = 1.0, b = 1.0, a = 1.0):
        self.owner = weakref.ref(owner)
        self._r = r
        self._g = g
        self._b = b
        self._a = a
        self.UpdateOwner()

    def UpdateOwner(self):
        owner = self.owner()
        if owner is not None:
            if owner.renderObject:
                owner.renderObject.color = (self._r,
                 self._g,
                 self._b,
                 self._a)

    @apply
    def r():
        doc = 'Red component of this color'

        def fget(self):
            return self._r

        def fset(self, value):
            self._r = value
            self.UpdateOwner()

        return property(**locals())

    @apply
    def g():
        doc = 'Green component of this color'

        def fget(self):
            return self._g

        def fset(self, value):
            self._g = value
            self.UpdateOwner()

        return property(**locals())

    @apply
    def b():
        doc = 'Blue component of this color'

        def fget(self):
            return self._b

        def fset(self, value):
            self._b = value
            self.UpdateOwner()

        return property(**locals())

    @apply
    def a():
        doc = 'Alpha component of this color'

        def fget(self):
            return self._a

        def fset(self, value):
            self._a = value
            self.UpdateOwner()

        return property(**locals())

    def SetRGB(self, r, g, b, a = 1.0):
        self.r = r
        self.g = g
        self.b = b
        self.a = a

    SetRGBA = SetRGB

    def GetRGB(self):
        return (self.r, self.g, self.b)

    def GetRGBA(self):
        return (self.r,
         self.g,
         self.b,
         self.a)

    def SetAlpha(self, a):
        self.a = a

    def GetAlpha(self):
        return self.a

    def GetHSV(self):
        return trinity.TriColor(*self.GetRGBA()).GetHSV()

    def SetHSV(self, h, s, v):
        c = trinity.TriColor()
        c.SetHSV(h, s, v)
        self.SetRGB(c.r, c.g, c.b)
